package djmil.cordacheckers.cordaclient;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatNoException;

import java.util.Arrays;
import java.util.Map;

import javax.naming.InvalidNameException;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import djmil.cordacheckers.cordaclient.dao.GameState;
import djmil.cordacheckers.cordaclient.dao.Rank;
import djmil.cordacheckers.cordaclient.dao.Stone;
import djmil.cordacheckers.user.HoldingIdentityResolver;

@SpringBootTest
public class RankingTests {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    final String player1 = "Kumar";
    final String player2 = "Bobik";

    final static Stone WHITE_MAN  = new Stone(Stone.Color.WHITE, Stone.Type.MAN);
    final static Stone WHITE_KING = new Stone(Stone.Color.WHITE, Stone.Type.KING);
    final static Stone BLACK_MAN  = new Stone(Stone.Color.BLACK, Stone.Type.MAN);
    final static Stone BLACK_KING = new Stone(Stone.Color.BLACK, Stone.Type.KING);

    final static Map<Integer, Stone> board1 =  Map.ofEntries(
        Map.entry(10, BLACK_KING),
        Map.entry(7, WHITE_KING)
    );

    @Test
    void testSurrender() throws InvalidNameException {
        final var hiCustodian  = holdingIdentityResolver.getCustodian();
        assertThat(hiCustodian).isNotNull();

        final Map<String, Rank> liderboard1 = cordaClient.fetchRanking(hiCustodian);

        final var hiWinner = holdingIdentityResolver.getByUsername(player1);
        final var hiLooser = holdingIdentityResolver.getByUsername(player2);
        final var winnerName = hiWinner.getName();
        final var losserName = hiLooser.getName();

        final GameState game = cordaClient.gameProposalCreate(hiWinner, hiLooser, Stone.Color.WHITE, 
            "GameBoard GLOBAL_RANKING surrender test");

        cordaClient.gameProposalAccept(hiLooser, game.uuid());
        cordaClient.gameBoardSurrender(hiLooser, game.uuid());

        final Map<String, Rank> liderboard2 = cordaClient.fetchRanking(hiCustodian);

        assertThat(liderboard1.get(winnerName).gamesWon()    +1 == liderboard2.get(winnerName).gamesWon()   );
        assertThat(liderboard1.get(winnerName).gamesPlayed() +1 == liderboard2.get(winnerName).gamesPlayed());
        assertThat(liderboard1.get(winnerName).gamesDraw()      == liderboard2.get(winnerName).gamesDraw()  );

        assertThat(liderboard1.get(losserName).gamesWon()       == liderboard2.get(losserName).gamesWon()   );
        assertThat(liderboard1.get(losserName).gamesPlayed() +1 == liderboard2.get(losserName).gamesPlayed());
        assertThat(liderboard1.get(losserName).gamesDraw()      == liderboard2.get(losserName).gamesDraw()  );
    }

    @Test
    void testVictory() throws InvalidNameException {
        final var hiCustodian  = holdingIdentityResolver.getCustodian();
        assertThat(hiCustodian).isNotNull();

        final Map<String, Rank> liderboard1 = cordaClient.fetchRanking(hiCustodian);

        final var hiWinner = holdingIdentityResolver.getByUsername(player2);
        final var hiLooser = holdingIdentityResolver.getByUsername(player1);
        final var winnerName = hiWinner.getName();
        final var losserName = hiLooser.getName();

        final GameState game = cordaClient.gameProposalCreate(hiWinner, hiLooser, Stone.Color.BLACK, 
            board1, "GameBoard GLOBAL_RANKING victory test");

        cordaClient.gameProposalAccept(hiLooser, game.uuid());
        cordaClient.gameBoardMove(hiWinner, game.uuid(), Arrays.asList(7, 14), null);

        final Map<String, Rank> liderboard2 = cordaClient.fetchRanking(hiCustodian);

        assertThat(liderboard1.get(winnerName).gamesWon()    +1 == liderboard2.get(winnerName).gamesWon()   );
        assertThat(liderboard1.get(winnerName).gamesPlayed() +1 == liderboard2.get(winnerName).gamesPlayed());
        assertThat(liderboard1.get(winnerName).gamesDraw()      == liderboard2.get(winnerName).gamesDraw()  );

        assertThat(liderboard1.get(losserName).gamesWon()       == liderboard2.get(losserName).gamesWon()   );
        assertThat(liderboard1.get(losserName).gamesPlayed() +1 == liderboard2.get(losserName).gamesPlayed());
        assertThat(liderboard1.get(losserName).gamesDraw()      == liderboard2.get(losserName).gamesDraw()  );
    }

    @Test
    void testDraw() throws InvalidNameException {
        final var hiCustodian  = holdingIdentityResolver.getCustodian();
        assertThat(hiCustodian).isNotNull();

        final var hiPlayer1 = holdingIdentityResolver.getByUsername(player1);
        final var hiPlayer2 = holdingIdentityResolver.getByUsername(player2);

        final GameState game = cordaClient.gameProposalCreate(hiPlayer1, hiPlayer2, Stone.Color.WHITE, 
            "GameBoard GLOBAL_RANKING draw test");

        cordaClient.gameProposalAccept(hiPlayer2, game.uuid());
        cordaClient.gameDrawRequest(hiPlayer2, game.uuid());

        final Map<String, Rank> liderboard1 = cordaClient.fetchRanking(hiCustodian);
        cordaClient.gameDrawAccept(hiPlayer1, game.uuid());
        final Map<String, Rank> liderboard2 = cordaClient.fetchRanking(hiCustodian);

        assertThat(liderboard1.get(player1).gamesWon()       == liderboard2.get(player1).gamesWon()   );
        assertThat(liderboard1.get(player1).gamesPlayed() +1 == liderboard2.get(player1).gamesPlayed());
        assertThat(liderboard1.get(player1).gamesDraw()   +1 == liderboard2.get(player1).gamesDraw()  );

        assertThat(liderboard1.get(player2).gamesWon()       == liderboard2.get(player2).gamesWon()   );
        assertThat(liderboard1.get(player2).gamesPlayed() +1 == liderboard2.get(player2).gamesPlayed());
        assertThat(liderboard1.get(player2).gamesDraw()   +1 == liderboard2.get(player2).gamesDraw()  );
    }

    @Test
    void testIndividualRanking() {
        final var hiCustodian  = holdingIdentityResolver.getByUsername(player2);

        assertThatNoException().isThrownBy( () -> 
            cordaClient.fetchRanking(hiCustodian)
        );
    }

}
